@@ -1,6 +1,7 @@
 import json
 import os
 import warnings
+from datetime import datetime, timedelta, time
 from pathlib import Path
 from typing import Any, Dict, Optional
 from xml.etree import ElementTree
@@ -38,6 +39,15 @@ class MarklogicResourceNotFoundError(MarklogicAPIError):
     pass
 
 
+class MarklogicResourceLockedError(MarklogicAPIError):
+    pass
+
+
+class MarklogicResourceUnmanagedError(MarklogicAPIError):
+    """Note: this exception may be raised if a document doesn't exist"""
+    pass
+
+
 class MarklogicCommunicationError(MarklogicAPIError):
     pass
 
@@ -51,7 +61,9 @@ class MarklogicApiClient:
         404: MarklogicResourceNotFoundError,
     }
     error_code_classes = {
-        'XDMP-DOCNOTFOUND': MarklogicResourceNotFoundError
+        'XDMP-DOCNOTFOUND': MarklogicResourceNotFoundError,
+        'XDMP-LOCKCONFLICT': MarklogicResourceLockedError,
+        'DLS-UNMANAGED': MarklogicResourceUnmanagedError,
     }
 
     default_http_error_class = MarklogicCommunicationError
@@ -153,8 +165,9 @@ class MarklogicApiClient:
             vars=json.dumps(vars),
             accept_header="application/xml",
         )
+
         if not response.text:
-            return ''
+            raise MarklogicNotPermittedError("The document is not published and show_unpublished was not set")
 
         multipart_data = decoder.MultipartDecoder.from_response(response)
         return multipart_data.parts[0].text
@@ -291,15 +304,22 @@ class MarklogicApiClient:
             accept_header="application/xml",
         )
 
-    def checkout_judgment(self, judgment_uri: str) -> requests.Response:
+    def checkout_judgment(self, judgment_uri: str, annotation="", expires_at_midnight=False) -> requests.Response:
         uri = f"/{judgment_uri.lstrip('/')}.xml"
         xquery_path = os.path.join(
             ROOT_DIR, "xquery", "checkout_judgment.xqy"
         )
         vars = {
-            "uri": uri
+            "uri": uri,
+            "annotation": annotation,
         }
 
+        if expires_at_midnight:
+            timeout = self.calculate_seconds_until_midnight()
+            vars["timeout"] = timeout
+        else:
+            vars["timeout"] = -1
+
         return self.eval(
             xquery_path,
             vars=json.dumps(vars),
@@ -321,6 +341,33 @@ class MarklogicApiClient:
             accept_header="application/xml",
         )
 
+    def get_judgment_checkout_status(self, judgment_uri: str) -> requests.Response:
+        uri = f"/{judgment_uri.lstrip('/')}.xml"
+        xquery_path = os.path.join(
+            ROOT_DIR, "xquery", "get_judgment_checkout_status.xqy"
+        )
+        vars = {
+            "uri": uri
+        }
+
+        return self.eval(
+            xquery_path,
+            vars=json.dumps(vars),
+            accept_header="application/xml",
+        )
+
+    def get_judgment_checkout_status_message(self, judgment_uri: str):
+        """Return the annotation of the lock or `None` if there is no lock."""
+        response = self.get_judgment_checkout_status(judgment_uri)
+        if not response.content:
+            return None
+        content = decoder.MultipartDecoder.from_response(response).parts[0].text
+        if content == "":
+            return None
+        response_xml = ElementTree.fromstring(content)
+        return response_xml.find("dls:annotation", namespaces={"dls": "http://marklogic.com/xdmp/dls"}).text
+
+
     def get_judgment_version(self, judgment_uri: str, version: int) -> requests.Response:
         uri = f"/{judgment_uri.lstrip('/')}.xml"
         xquery_path = os.path.join(
@@ -584,10 +631,22 @@ class MarklogicApiClient:
             accept_header="application/xml",
         )
 
+    def calculate_seconds_until_midnight(self, now=None):
+        """
+        Get timedelta until end of day on the datetime passed, or current time.
+        https://stackoverflow.com/questions/45986035/seconds-until-end-of-day-in-python
+        """
+        if not now:
+            now = datetime.now()
+        tomorrow = now + timedelta(days=1)
+        difference = datetime.combine(tomorrow, time.min) - now
+
+        return difference.seconds
+
 
 api_client = MarklogicApiClient(
-    host=env("MARKLOGIC_HOST"),
-    username=env("MARKLOGIC_USER"),
-    password=env("MARKLOGIC_PASSWORD"),
+    host=env("MARKLOGIC_HOST", default=None),
+    username=env("MARKLOGIC_USER", default=None),
+    password=env("MARKLOGIC_PASSWORD", default=None),
     use_https=env("MARKLOGIC_USE_HTTPS", default=False),
-)
\ No newline at end of file
+)
